@@ -1,9 +1,9 @@
 extern crate rfd;
 extern crate sixtyfps;
 
-use num_traits::{FromPrimitive, ToPrimitive};
+use num_traits::FromPrimitive;
 use rfd::FileDialog;
-use sixtyfps::{Model, ModelHandle, SharedString};
+use sixtyfps::{Model, SharedString};
 use std::collections::HashMap;
 use std::rc::Rc;
 use std::sync::Mutex;
@@ -17,7 +17,13 @@ use crate::persistence::json::{get_project_filename, get_settings_filename};
 use crate::persistence::settings::Settings;
 use crate::synchronize::Synchronizer;
 
-sixtyfps::include_modules!();
+const MAX_SIMILARS: usize = 5;
+
+#[allow(clippy::all)]
+mod generated_code {
+    sixtyfps::include_modules!();
+}
+pub use generated_code::*;
 
 type ImagesModelMap = HashMap<usize, usize>;
 
@@ -61,7 +67,9 @@ impl MainWindow {
         let synchronizer = Synchronizer::new(item_list.clone(), &image_sieve);
 
         // Start synchronization in a background thread
-        synchronizer.synchronize(&settings.source_directory);
+        synchronizer.synchronize(&settings.source_directory, settings.clone());
+        let mut cache = ImageCache::new();
+        cache.restrict_size(1600, 1000);
 
         let main_window = Self {
             window: image_sieve,
@@ -70,7 +78,7 @@ impl MainWindow {
             similar_items_model: Rc::new(sixtyfps::VecModel::<SortImage>::default()),
             items_model_map: Rc::new(RefCell::new(HashMap::new())),
             events_model: event_list_model,
-            image_cache: Rc::new(ImageCache::new()),
+            image_cache: Rc::new(cache),
         };
 
         // Set initial values
@@ -78,21 +86,7 @@ impl MainWindow {
         main_window
             .window
             .set_window_title(SharedString::from("ImageSieve v") + version);
-        main_window
-            .window
-            .set_source_directory(SharedString::from(settings.source_directory));
-        main_window
-            .window
-            .set_target_directory(SharedString::from(settings.target_directory));
-        let commit_index = ToPrimitive::to_i32(&settings.commit_method).unwrap();
-        main_window.window.set_commit_method(commit_index);
-        let values: ModelHandle<SharedString> = main_window
-            .window
-            .global::<CommitMethodValues>()
-            .get_values();
-        main_window
-            .window
-            .set_commit_method_value(values.row_data(commit_index as usize));
+        settings.to_window(&main_window.window);
 
         // Set model references
         main_window
@@ -119,12 +113,7 @@ impl MainWindow {
         self.window.run();
 
         // Save settings when program exits
-        let settings = Settings {
-            source_directory: self.window.get_source_directory().to_string(),
-            target_directory: self.window.get_target_directory().to_string(),
-            commit_method: FromPrimitive::from_i32(self.window.get_commit_method())
-                .unwrap_or(CommitMethod::Copy),
-        };
+        let settings = Settings::from_window(&self.window);
         JsonPersistence::save(get_settings_filename(), &settings);
 
         // and save item list
@@ -220,7 +209,8 @@ impl MainWindow {
 
                     // Synchronize in a background thread
                     window_weak.unwrap().set_loading(true);
-                    synchronizer.synchronize(source_path);
+                    synchronizer
+                        .synchronize(source_path, Settings::from_window(&window_weak.unwrap()));
 
                     window_weak
                         .unwrap()
@@ -321,6 +311,18 @@ impl MainWindow {
                 opener::open(item_list.items[i as usize].get_path()).ok();
             }
         });
+
+        self.window.on_recheck_similarities({
+            // Browse source was clicked, select new path
+            let window_weak = self.window.as_weak();
+            let synchronizer = Synchronizer::new(self.item_list.clone(), &self.window);
+
+            move || {
+                // Synchronize in a background thread
+                window_weak.unwrap().set_calculating_similarities(true);
+                synchronizer.synchronize("", Settings::from_window(&window_weak.unwrap()));
+            }
+        });
     }
 }
 
@@ -395,6 +397,7 @@ fn synchronize_images_model(
         let sort_image_struct = SortImage {
             image,
             take_over: item.get_take_over(),
+            text: get_item_text(*item_index, item_list),
         };
         similar_items_model.push(sort_image_struct);
         item_model_map.insert(model_index, *item_index);
@@ -404,7 +407,10 @@ fn synchronize_images_model(
     add_item(&selected_item_index);
 
     for image_index in similars {
-        add_item(image_index);
+        // TODO: This should be done in a different thread instead of limiting this here
+        if similar_items_model.row_count() <= MAX_SIMILARS {
+            add_item(image_index);
+        }
     }
 
     // Prefetch next two images
@@ -427,24 +433,17 @@ fn synchronize_images_model(
         .unwrap()
         .set_current_image(similar_items_model.row_data(0));
     window.unwrap().set_current_image_index(0);
-    window
-        .unwrap()
-        .set_num_images(similar_items_model.row_count() as i32);
+}
 
-    let item = &item_list.items[selected_item_index];
-    let mut item_text = item.get_item_string(&String::from(""));
-    let item_size = item.get_size() / 1024;
-    let item_date = item.get_date_str();
+pub fn get_item_text(index: usize, item_list: &ItemList) -> SharedString {
+    let item = &item_list.items[index];
     let event = item_list.get_event(item);
     let event_str = if let Some(event) = event {
         event.name.as_str()
     } else {
         ""
     };
-    item_text += format!(" - {}, {} KB {}", item_date, item_size, event_str).as_str();
-    window
-        .unwrap()
-        .set_current_image_text(SharedString::from(item_text));
+    SharedString::from(item.get_text(event_str))
 }
 
 pub fn commit(item_list: &ItemList, window_weak: sixtyfps::Weak<ImageSieve>) {
